<?php

namespace Drupal\rest_api_access_token\Authentication\Provider;

use Drupal\Core\Authentication\AuthenticationProviderInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\rest_api_access_token\Exception\AccessDeniedException;
use Drupal\rest_api_access_token\Exception\InvalidRequestIdException;
use Drupal\rest_api_access_token\Exception\TokenNotFoundException;
use Drupal\rest_api_access_token\Repository\TokenRepository;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\Request;

class AccessTokenProvider implements AuthenticationProviderInterface
{
    /** @var TokenRepository */
    protected $tokenRepository;

    /** @var ConfigFactoryInterface */
    protected $configFactory;

    /**
     * AccessTokenProvider constructor.
     * @param TokenRepository $tokenRepository
     * @param ConfigFactoryInterface $configFactory
     */
    public function __construct(TokenRepository $tokenRepository, ConfigFactoryInterface $configFactory)
    {
        $this->tokenRepository = $tokenRepository;
        $this->configFactory = $configFactory;
    }

    /**
     * {@inheritdoc}
     */
    public function applies(Request $request)
    {
        $token = $request->headers->get('X-AUTH-TOKEN');
        return !empty($token);
    }

    /**
     * @param Request $request
     * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Session\AccountInterface|User|null
     * @throws \Exception
     */
    public function authenticate(Request $request)
    {
        $publicToken = (string)$request->headers->get('X-AUTH-TOKEN');
        $signature = (string)$request->headers->get('X-AUTH-SIGNATURE');
        $requestId = (string)$request->headers->get('REQUEST-ID');
        $path = $request->getPathInfo();

        if (empty($publicToken)) {
            throw new AccessDeniedException(t('Empty X-AUTH-TOKEN'));
        }

        if ($this->configFactory->get('rest_api_access_token.config')->get('cache_endpoints')) {
            if (empty($requestId)) {
                throw new InvalidRequestIdException(t('Empty REQUEST-ID'));
            }
        }

        try {
            $token = $this->tokenRepository->getByPublicToken($publicToken);
        } catch (TokenNotFoundException $exception) {
            throw new AccessDeniedException(t('Invalid X-AUTH-TOKEN'));
        }

        $user = User::load($token->getUserId());
        if (!$user || !$user->isActive()) {
            throw new AccessDeniedException(t('Invalid user account'));
        }

        if ($this->configFactory->get('rest_api_access_token.config')->get('signature_verification')) {
            $value = $publicToken . '|' . $requestId . '|' . $path . '|' . base64_encode($request->getContent()) . '|' . $token->getSecret();
            if ($signature !== hash('sha256', $value)) {
                throw new AccessDeniedException(t('Invalid X-AUTH-SIGNATURE'));
            }
        }

        return $user;
    }
}
